#ifndef __TADASUPERSAMPRT_HH
#define __TADASUPERSAMPRT_HH
//
// Project : lib3D
// File    : TAdaSuperSampRT.hh
// Purpose : class for raytracing by adaptive super sampling
// Author  : Ronald Kriemann
//
// arch-tag: df6dbee9-cf4f-498e-9e2a-bd3651398791
//

#include "TSuperSamplingRT.hh"

namespace RGL
{

//
// uses single ray for all pixels and super sampling
// for regions with a high contrast
//
class TAdaSuperSampRT : public TSuperSamplingRT
{
protected:
    // minimal contrast between two pixels
    real  _min_contrast;
    
public:
    ///////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TAdaSuperSampRT ();

    virtual ~TAdaSuperSampRT () {}

    ///////////////////////////////////////////////
    //
    // access local data
    //

    virtual void set_contrast ( real c ) { _min_contrast = c; }
    
    ///////////////////////////////////////////////
    //
    // rendering
    //

    // render part of image
    virtual void render_tile ( const TTileDesc & tile,
                               TImage          & img,
                               TProgressBar    * bar = NULL ) const;
    
protected:
    ///////////////////////////////////////////////
    //
    // determine contrast 
    //

    virtual bool exceeds_contrast ( const TColour * pix,
                                    const TColour * upper,
                                    const TColour * lower,
                                    const TColour * left,
                                    const TColour * right ) const;

    virtual real colour_dist ( const TColour & c1,
                               const TColour & c2 ) const
    {
        real  r = c1.red()   - c2.red();
        real  g = c1.green() - c2.green();
        real  b = c1.blue()  - c2.blue();
        
        return r*r + g*g + b*b;
    }
};

} // namespace

#endif  // __TADASUPERSAMPRT_HH
