#ifndef __TADAPTIVERT_HH
#define __TADAPTIVERT_HH
//
// Project : lib3D
// File    : TAdaptiveRT.hh
// Purpose : class for raytracing by adaptive sampling
// Author  : Ronald Kriemann
//
// arch-tag: 62e470ba-7738-4aeb-b3f2-4f24a43152de
//

#include "TRaytracer.hh"

namespace RGL
{

//
// forward definitions
//

class TPixel;

//
// computes colour of pixel by adaptively subdividing pixel
//
class TAdaptiveRT : public TRaytracer
{
protected:
    // maximal contrast between two colours
    real  _max_contrast;

    // maximal level of adaptivity
    uint  _max_ada_lvl;
    
public:
    ///////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TAdaptiveRT ();

    virtual ~TAdaptiveRT () {}

    ///////////////////////////////////////////////
    //
    // access local data
    //

    virtual void set_max_contrast ( real c ) { _max_contrast = c; }
    virtual void set_max_level    ( uint l ) { _max_ada_lvl  = l; }
    
    ///////////////////////////////////////////////
    //
    // rendering
    //

    // render part of image
    virtual void render_tile ( const TTileDesc & tile,
                               TImage          & img,
                               TProgressBar    * bar = NULL ) const;
    
protected:
    ///////////////////////////////////////////////
    //
    // adaptivity methods
    //

    // render colour for a pixel
    void render_pixel ( TPixel * pixel ) const;

    // compare pixel with neighbours
    void compare_pixel ( TPixel * pixel,
                         TPixel * upper,
                         TPixel * lower,
                         TPixel * left,
                         TPixel * right,
                         uint     lvl ) const;

    // distance between two colours (squared distant)
    real colour_dist ( const TColour & c1, const TColour & c2 ) const
    {
        real  r = c1.red()   - c2.red();
        real  g = c1.green() - c2.green();
        real  b = c1.blue()  - c2.blue();
        
        return r*r + g*g + b*b;
    }
};

} // namespace

#endif  // __TADAPTIVERT_HH
