#ifndef __TAMBIENT_HH
#define __TAMBIENT_HH
//
// Project : lib3D
// File    : TAmbient.hh
// Purpose : class for ambient illumination
// Author  : Ronald Kriemann
//
// arch-tag: 50b40ab6-32ae-4b57-b6da-36ff454f8e1e
//

#include "TLight.hh"

namespace RGL
{

//
// 
//
class TAmbient : public TLight
{
protected:
    // colour of ambient light
    TColour   _colour;
    
public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TAmbient ( const TColour & col ) : _colour( col ) {}

    virtual ~TAmbient () {}
    
    //////////////////////////////////////////////
    //
    // illumination methods
    //

    // calculate resulting colour at point with
    // normal and overwrite given base-colour
    virtual void illuminate ( const THPDesc & hp,
                              const TVector & origin,
                              TColour       & colour ) const
    {
        // independent of point and normal
        real     ambient = hp.object()->texture()->ambient();
        TColour  col;

        hp.object()->texture()->colour( hp.hitpoint(), col );
        
        colour.set_rgb( col.red()   * _colour.red()   * ambient,
                        col.green() * _colour.green() * ambient,
                        col.blue()  * _colour.blue()  * ambient );
    }

    // test if light is visible from given point
    virtual bool is_visible ( const TVector    & point,
                              const TIntersect & intersect ) const
    { return true; }

    //////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TAmbient * clone () const { return new TAmbient( _colour ); }
    virtual TAmbient * copy  () const { return (TAmbient*) TLight::copy(); }
};

} // namespace

#endif  // __TAMBIENT_HH
