//
// Project : misc
// File    : TArray.hh
// Purpose : class for a dyn. array of templates
// Author  : Ronald Kriemann
//
// arch-tag: 9e6547c4-d41d-438a-ba4c-f44eb5fa8518
//

//
// constructor with default size
//
template <class T> 
TArray<T>::TArray ( uint size )
        : _size( size )
{
    if ( _size > 0 )
        _data = new T[_size];
    else
        _data = NULL;
}

//
// set a new size but keep the old elements
//
template <class T> 
void 
TArray<T>::set_size ( uint  n )
{
    if ( n == _size )
        return;

    //
    // create new array and copy old data
    // (to be done: n <= 0)
    //
    
    T   * tmp = NULL;
    T   * p1, * p2;

    if (n > 0)
        tmp = new T[n];

    p1 = tmp;
    p2 = _data;
    
    if ( _size > n )
    {
        for ( uint i = 0; i < n; i++ )
            *p1++ = *p2++;
    }// if
    else
    {
        for ( uint i = 0; i < _size; i++ )
            *p1++ = *p2++;

        // the rest of the new data is undefinded
    }// else

    delete[] _data;

    _data = tmp;
    _size = n;
}

//
// copy operator
//
template <class T> 
TArray<T> & 
TArray<T>::operator = ( const TArray<T> & vec )
{
	if (vec._size == _size)
	{
		T * p1 = _data;
		T * p2 = vec._data;

		for ( uint i = 0; i < _size; i++)
			*p1++ = *p2++;
	}// if
	else 
	{
		_size = vec._size;

		// set new size but no need for keeping old elements
		if (_data != NULL)
            delete[] _data;

        if (_size > 0)
            _data = new T[_size];
        else
            _data = NULL;

		if (_size > 0)
		{
			T * p1 = _data;
			T * p2 = vec._data;
            
			for ( uint i = 0; i < _size; i++)
				*p1++ = *p2++;
		}// if
	}// else
    
	return (*this);
}

template <class T>
TArray<T> &
TArray<T>::copy ( T * data, uint size )
{
    if ( size < 0 )
        size = 0;
    
    set_size( size );

    for ( uint i = 0; i < size; i++ )
        _data[i] = data[i];

    return *this;
}

