#ifndef __TARRAY_HH
#define __TARRAY_HH
//
// Project : misc
// File    : TArray.hh
// Purpose : class for a dyn. array of templates
// Author  : Ronald Kriemann
//
// arch-tag: c157d7e8-53cb-4f25-bffa-a1f1a78e076b
//

#include <assert.h>

#include <iostream>

#include "defines.hh"

namespace RGL
{

//
// class for a template-vector
//
template <class T> 
class TArray
{
protected:
    T     * _data;         // the array with the stored elements
    uint    _size;         // number of the element with the max. index

public:
    /////////////////////////////////////////
    //
    // constructor and destructor
    //

    TArray ( uint size = 0 );
    TArray ( const TArray<T> & vec ) : _data(NULL), _size(0) { (*this) = vec; }

    ~TArray () { if (_data) delete[] _data; }

    /////////////////////////////////////////
    //
    // manipulate array
    //
    
    // get/set size of the vector
    uint size     () const { return _size; }
    void set_size ( uint  n );
    
    // copy operator
    TArray<T> & operator = ( const TArray<T> & vec );

    /////////////////////////////////////////
    //
    // access array
    //
    
    T &       operator [] ( uint  i )       { assert(i < _size ); return _data[i]; }
    const T & operator [] ( uint  i ) const { assert(i < _size ); return _data[i]; }

    // return c-style array
    T *       c_array ()       { return _data; }
    const T * c_array () const { return _data; }

    // copy c-style array
    TArray &  copy ( T * data, uint size );
};

#include "TArray.cc"

} // namespace

#endif            // __TARRAY_HH
