#ifndef __TCSG_HH
#define __TCSG_HH
//
// Project : lib3D
// File    : TCSG.hh
// Purpose : class for handling constructive solid geometry
// Author  : Ronald Kriemann
//
// arch-tag: a21f66b0-650e-40be-bac7-8c5a65a570a7
//

#include "TGraphObject.hh"

namespace RGL
{

//
// combination type
//
typedef enum { CSG_UNION, CSG_INTERSECTION, CSG_DIFFERENCE } csg_type_t;

//
// supports union, intersection and difference
//
class TCSG : public TGraphObject
{
protected:
    // combined objects  ( left \cup right, left \cap right or left \ right)
    TGraphObject  * _left;
    TGraphObject  * _right;

    // type of combination
    csg_type_t      _type;

public:
    //////////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TCSG ( TGraphObject * left,
           TGraphObject * right,
           csg_type_t     type );

    virtual ~TCSG ();
    
    //////////////////////////////////////////////////
    //
    // raytracing methods
    //

    // compute intersection with ray
    // - return true if intersected, else false
    virtual bool intersect ( const TRay & ray, real & dist ) const;

    // compute normal at point
    virtual bool calc_normal ( const TVector & point,
                               TVector       & normal ) const;

    //////////////////////////////////////////////////
    //
    // misc. methods
    //

    // store bounding box defined by xy, xz and yz planes
    // in given vectors
    // - return false of bb is not defined (e.g. infinite object)
    virtual bool bounding_box ( TVector & min, TVector & max ) const;

    // print object
    virtual void print () const;
    
    //////////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TGraphObject * clone () const;
    virtual TGraphObject * copy  () const;
};

}// namespace

#endif  // __TCSG_HH
