#ifndef __TCOLOURLOOKUP_HH
#define __TCOLOURLOOKUP_HH
//
// Project : lib3d
// File    : TColourLookup.hh
// Purpose : class for converting names to rgb-values
// Author  : Ronald Kriemann
//
// arch-tag: c8183774-6bd1-466a-9cdf-026225fdaa3f
//

#include "defines.hh"
#include "TString.hh"

namespace RGL
{

//
// forward decl. for the search tree
//
class TTSTree;

//
// this class is a singleton
//
class TColourLookup
{
protected:
    // a search-tree holding the data
    TTSTree  * _colour_db;
    
public:
    /////////////////////////////////////
    //
    // constructor
    //

    TColourLookup ();

    /////////////////////////////////////
    //
    // access the colours
    //

    ulong name_to_rgb ( const TString & name ) const;

    /////////////////////////////////////
    //
    // read colours
    //

    // from internal database
    void read_internal ();
    
    // from X11 - rgb.txt
    void read_x11_colours ( const TString & filename );
};

// lookup-instance
extern TColourLookup  colour_lookup;

// for ease of use
inline ulong
name_to_rgb ( const TString & name )
{
    return colour_lookup.name_to_rgb( name );
}

} // namespace

#endif
