#ifndef __THPDESC_HH
#define __THPDESC_HH
//
// Project : lib3D
// File    : THPDesc.hh
// Purpose : class holding information about the hit-surface
// Author  : Ronald Kriemann
//
// arch-tag: 3015547b-09e2-4045-bad9-b9f222b16ff2
//

#include <stdio.h>

#include "TVector.hh"

namespace RGL
{

// forward decl.
class TGraphObject;

class THPDesc
{
protected:
    // data: distance, hitpoint, object and normal
    real           _dist;
    TGraphObject * _hitobject;
    TVector        _hitpoint;
    TVector        _normal;

public:
    ////////////////////////////////////////////////
    //
    // constructor and destructor
    //

    THPDesc ( TGraphObject * ho, real dist )
            : _dist(dist), _hitobject( ho )
    {
        if ( ho == NULL )
            std::cout << "(THPDesc) got NULL object in constructor" << std::endl;
    }

    THPDesc ( const THPDesc & hp ) { *this = hp; }
        
    ////////////////////////////////////////////////
    //
    // access data
    //

    real           distance  () { return _dist; }
    TGraphObject * object    () { return _hitobject; }
    TVector      & hitpoint  () { return _hitpoint; }
    TVector      & normal    () { return _normal; }

    real                 distance  () const { return _dist; }
    const TGraphObject * object    () const { return _hitobject; }
    const TVector      & hitpoint  () const { return _hitpoint; }
    const TVector      & normal    () const { return _normal; }

    ////////////////////////////////////////////////
    //
    // misc.
    //

    THPDesc & operator = ( const THPDesc & hp )
    {
        _dist      = hp._dist;
        _hitobject = hp._hitobject;
        _hitpoint  = hp._hitpoint;
        _normal    = hp._normal;
        return *this;
    }
};

}// namespace

#endif  // __THPDESC_HH
