#ifndef __TIMAGEIO_HH
#define __TIMAGEIO_HH
//
// Project : lib3D
// File    : TImageIO.hh
// Purpose : classes for image file-io
// Author  : Ronald Kriemann
//
// arch-tag: c08596b2-ec9d-48f1-b1c5-6a0617463bfd
//

#include "TImage.hh"
#include "TFile.hh"

namespace RGL
{

//////////////////////////////////////////////////////////////
//
// handler for transparent image loading/saving
//
//////////////////////////////////////////////////////////////

class TImgIOHandler
{
protected:
    // gamma correction factor
    real  _gamma;
    
public:
    /////////////////////////////////////////
    //
    // constructor and destructor
    //

    TImgIOHandler ( real gamma = 1.0 )
            : _gamma(gamma)
    {}

    virtual ~TImgIOHandler () {}

    // access gamma correction
    real gamma     () const       { return _gamma; }
    void set_gamma ( real gamma ) { _gamma = gamma; }
    
    /////////////////////////////////////////
    //
    // basic functionality: load and save
    //

    virtual int load ( const TString & fname, TImage       & img ) const;
    virtual int save ( const TString & fname, const TImage & img ) const;
};

//////////////////////////////////////////////////////////////
//
// Image-IO classes
//
//////////////////////////////////////////////////////////////

//
// baseclass
//
class TImageIO
{
protected:
    // gamma correction factor
    real  _gamma;
    
public:
    /////////////////////////////////////////
    //
    // constructor and destructor
    //

    TImageIO ( real gamma = 1.0 )
            : _gamma(gamma)
    {}

    virtual ~TImageIO () {}

    // access gamma correction
    real gamma     () const       { return _gamma; }
    void set_gamma ( real gamma ) { _gamma = gamma; }
    
    /////////////////////////////////////////
    //
    // basic functionality: load and save
    //

    virtual int load ( const TString & fname, TImage       & img ) const = 0;
    virtual int save ( const TString & fname, const TImage & img ) const = 0;
};

//
// class for PNG-files
//
class TPNGImgIO : public TImageIO
{
public:

    /////////////////////////////////////////
    //
    // constructor and destructor
    //

    TPNGImgIO ( real gamma = 1.0 )
            : TImageIO( gamma )
    {}

    virtual ~TPNGImgIO () {}

    /////////////////////////////////////////
    //
    // basic functionality: load and save
    //

    virtual int load ( const TString & fname, TImage       & img ) const;
    virtual int save ( const TString & fname, const TImage & img ) const;
};

//
// class for JPEG-files
//
class TJPEGImgIO : public TImageIO
{
public:

    /////////////////////////////////////////
    //
    // constructor and destructor
    //

    TJPEGImgIO ( real gamma = 1.0 )
            : TImageIO( gamma )
    {}

    virtual ~TJPEGImgIO () {}

    /////////////////////////////////////////
    //
    // basic functionality: load and save
    //

    virtual int load ( const TString & fname, TImage       & img ) const;
    virtual int save ( const TString & fname, const TImage & img ) const;
};

//
// class for PNM-files
//
class TPNMImgIO : public TImageIO
{
public:

    /////////////////////////////////////////
    //
    // constructor and destructor
    //

    TPNMImgIO ( real gamma = 1.0 )
            : TImageIO( gamma )
    {}

    virtual ~TPNMImgIO () {}

    /////////////////////////////////////////
    //
    // basic functionality: load and save
    //

    virtual int load ( const TString & fname, TImage       & img ) const;
    virtual int save ( const TString & fname, const TImage & img ) const;

protected:
    //
    // special functions for individual formats
    //

    int save_ppm ( TFile & file, const TImage & img ) const;
};

}// namespace

#endif // __TIMAGEIO_HH
