#ifndef __TMULTIREF_HH
#define __TMULTIREF_HH
//
// Project : H-Matrix
// File    : TMultiRef.hh
// Purpose : class with multi-reference-ability
// Author  : Ronald Kriemann
//
// arch-tag: 3ce77274-8a90-4829-b44e-8d3cf6ef95cd
//

#include <iostream>

namespace RGL
{

//
// multiple references for C++-objects (intelligent pointers)
//
// this class is baseclass for objects, which are referenced by
// more than one objects and must only be deleted if no reference
// exists
//
class TMultiRef
{
public:
    // counter for the number of references
    // (signed to handle errors, e.g. to many del_ref())
    int _references;

public:
    /////////////////////////////////////////////////
    //
    // constructor and destructor
    //
    
    TMultiRef ()                      : _references( 0 ) {}
    TMultiRef ( const TMultiRef & r ) : _references( 0 ) {}
    
    virtual ~TMultiRef ()
    { 
        if (_references > 0) 
            std::cerr << "(TMultiRef) ~TMultiRef : "
                      << "deleting an object with " << _references
                      << " references" << std::endl; 
    }

    /////////////////////////////////////////////////
    //
    // manage reference numbers
    //

    // add/del a reference
    void add_ref () { ++_references; }
    bool del_ref () { return (-- _references) == 0; }

    // check if reference is set
    bool has_ref () { return ! _references;}
    
    // do not copy number of referencess with an object
    TMultiRef & operator = ( const TMultiRef & ) { return *this; }

    // decrement reference counter and delete object if no ref left
    bool free_ref ()
    {
        if ((-- _references) > 0)
            return false;
        
        // looks like suicide, doesn't it :)
        delete this;
        
        return true;
    }
};

} // namespace

#endif  // __TMULTIREF_HH
