#ifndef __TNIEDERREITER_HH
#define __TNIEDERREITER_HH
//
// Project : misc
// File    : TNiederreiter.hh
// Purpose : class for a Niederreiter sequence generator
// Author  : Ronald Kriemann
//
// arch-tag: c6e07ddc-6ef3-46cc-9eb5-2ece6bd3383a
//

#include "defines.hh"
#include "TVector.hh"
#include "TArray.hh"

namespace RGL
{

//
// defines for the Niederreiter sequence
//
#define NIED2_CHARACTERISTIC  2
#define NIED2_BASE            2

#define NIED2_MAX_DIMENSION   6
#define NIED2_MAX_PRIM_DEGREE 5
#define NIED2_MAX_DEGREE      50

#define NIED2_BIT_COUNT       30
#define NIED2_NBITS           (NIED2_BIT_COUNT+1)

//
// generate numbers defined by Niederreiter sequences
//
// Implementation stems from the GNU Scientific Library
// (just restricted to 6 dimensions)
//
class TNiederreiter
{
protected:
    // dimension of the Niederreiter sequence
    uint    _dim;

    // data for the Niederreiter sequence
    uint    _sequence_count;
    int     _cj[ NIED2_NBITS ][ NIED2_MAX_DIMENSION ];
    int     _nextq[ NIED2_MAX_DIMENSION ];
    
public:
    ////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TNiederreiter ( uint dim ) { init( dim ); }

    ////////////////////////////////////////////
    //
    // access Niederreiter sequence
    //

    // return next number in sequence
    void next ( TArray< double > & val, double max = 1.0 );

    // return next number in sequence (special version for dim = 3)
    void next ( TVector & v, double max = 1.0 );
    
    // initialise Niederreiter sequence
    void init ( uint dim );
};

}// namespace

#endif  // __TNIEDERREITER_HH
