#ifndef __TORTHOCAMERA_HH
#define __TORTHOCAMERA_HH
//
// Project : lib3D
// File    : TOrthoCamera.hh
// Purpose : class representing a camera (viewing plane etc)
// Author  : Ronald Kriemann
//
// arch-tag: 03bb251f-426e-43cb-8058-482c02f1f517
//

#include "TCamera.hh"

namespace RGL
{

//
// orthographic camera with rays parallel to the
// direction of the camera
//
class TOrthoCamera : public TCamera
{
protected:
    // normalised vector pointing into the
    // direction of the camera
    TVector   _dir;
    
public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TOrthoCamera ();

    virtual ~TOrthoCamera ();

    //////////////////////////////////////////////
    //
    // setup ray corresponding to pixel
    // coordinates (i,j), i,j \in [0,1]
    //

    virtual void set_rays ( TArray< TRay > & rays, real i, real j ) const
    {
        rays.set_size( 1 );

        rays[0].new_id();
        rays[0].set_initial();
        rays[0].origin().set( 1.0, _ulc ).add( i, _vp_right, j, _vp_down );
        rays[0].direction() = _dir;
    }
        
private:

    //
    // update internal data
    //
    
    virtual void update ();
};

} // namespace

#endif  // __TORTHOCAMERA_HH
