#ifndef __TPLANE_HH
#define __TPANE_HH
//
// Project : lib3D
// File    : TPlane.hh
// Purpose : class for a (infinite) plane
// Author  : Ronald Kriemann
//
// arch-tag: a7e93db3-6b1d-4a8f-873a-40fab7c013a4
//

#include "TGraphObject.hh"

namespace RGL
{

//
// planes are define by normal-direction and distance from origin,
// default plane is x-z-plane
//
class TPlane : public TGraphObject
{
protected:
    // plane defined by normal and distance from origin
    TVector  _normal;
    real     _d;

public:
    /////////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TPlane ();
    
    TPlane ( const TVector & x0,
             const TVector & x1,
             const TVector & x2 );

    TPlane ( const TVector & normal,
             real            dist );

    ~TPlane () {}

    //////////////////////////////////////////////////
    //
    // access internal data
    //

    const TVector & normal   () const { return _normal; }
    real            distance () const { return _d; }
    
    //////////////////////////////////////////////////
    //
    // raytracing methods
    //

    // compute intersection with ray
    // - return true if intersected, else false
    virtual bool intersect ( const TRay & ray, real & dist ) const;

    // compute normal at point
    virtual bool calc_normal ( const TVector & point,
                               TVector       & normal ) const;

    //////////////////////////////////////////////////
    //
    // misc. methods
    //

    // store bounding box defined by xy, xz and yz planes
    // in given vectors
    virtual bool bounding_box ( TVector & min, TVector & max ) const { return false; }

    // print object
    virtual void print () const;
    
    //////////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TPlane * clone () const { return new TPlane; }
    virtual TPlane * copy  () const;
};

} // namespace

#endif  // __TPLANE_HH
