#ifndef __TPOINTLIGHT_HH
#define __TPOINTLIGHT_HH
//
// Project : lib3D
// File    : TPointLight.hh
// Purpose : class for a point-light (undirectional)
// Author  : Ronald Kriemann
//
// arch-tag: 879cd781-c64a-45f2-9e75-dc6e025106f9
//

#include "TLight.hh"

namespace RGL
{

//
// 
//
class TPointLight : public TLight
{
protected:
    // position of light
    TVector         _pos;
    
    // colour of light
    TColour         _colour;

    // last object, we hit
    mutable TGraphObject  * _last_obj;

public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TPointLight ( const TVector & pos,
                  const TColour & col )
            : _pos( pos ), _colour( col ), _last_obj( NULL )
    {}

    virtual ~TPointLight () {}
    
    //////////////////////////////////////////////
    //
    // access internal data
    //

    const TVector & position () const { return _pos; }
    
    //////////////////////////////////////////////
    //
    // illumination methods
    //

    // calculate resulting colour at point with
    // normal and overwrite given base-colour
    virtual void illuminate ( const THPDesc & hp,
                              const TVector & origin,
                              TColour       & colour ) const;

    // test if light is visible from given point
    virtual bool is_visible ( const TVector    & point,
                              const TIntersect & intersect ) const;

    //////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TPointLight * clone () const { return new TPointLight( _pos, _colour ); }
    virtual TPointLight * copy  () const { return (TPointLight*) TLight::copy(); }
};

} // namespace

#endif  // __TPOINTLIGHT_HH
