#ifndef __TPROGRESSBAR_HH
#define __TPROGRESSBAR_HH
//
// Project : PHI
// File    : TProgressBar.hh
// Purpose : class for showing the progress in various formats
// Author  : Ronald Kriemann
//
// arch-tag: da9fc28c-7c10-4356-8675-f9c462e1663b
//

#include "defines.hh"
#include "TString.hh"
#include "TTimer.hh"

namespace RGL
{

//
// print status in various styles
//
class TProgressBar
{
protected:
    // minimal, maximal and current value of progress
    real         _min, _max, _curr;

    // last value in output
    real         _old_val;
    
    // length of text-line
    uint         _text_len;

    // timer for printing ETA
    TTimer       _timer;

    // output prefix
    TString      _prefix;

    // is progress bar initialised
    bool         _is_init;
    
public:
    //
    // constructor and destructor
    //
    
    TProgressBar ( real min  = 0.0,
                   real max  = 100.0,
                   real curr = 0.0 );

    ~TProgressBar ();

    void set_prefix ( const TString & prefix ) { _prefix = prefix; }

    bool is_initialised () const { return _is_init; }
    
    //
    // change status
    //

    // initialise status
    void init ( real min, real max, real curr );

    // advance progress
    void advance ( real f );

    // finish output
    void finish ();

protected:
    // print progress
    void print ();
};

}// namespace

#endif  // __TPROGRESSBAR_HH
