#ifndef __TRNG_HH
#define __TRNG_HH
//
// Project : misc
// File    : TRNG.hh
// Purpose : class for a random-number-generator
// Author  : Ronald Kriemann
//
// arch-tag: f07178e7-450a-449b-bc4a-76cb1ca2fda3
//

#include "defines.hh"
#include "TVector.hh"
#include "TArray.hh"

namespace RGL
{

//
// pseudo random number generator
// - if seed is 0, time() is used
//
class TRNG
{
protected:
    // dimension of the random sequence
    uint            _dim;
    
    // state of the RNG (624 == N from TRNG.cc)
    unsigned long   _state[624];
    int             _left;
    unsigned long * _next;

public:
    ////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TRNG ( uint dim = 1, ulong seed = 5489UL ) { init( dim, seed ); }

    ////////////////////////////////////////////
    //
    // access rng
    //

    // return next element in sequence
    void next ( TArray< double > & val, double max = 1.0 )
    {
        val.set_size( _dim );
        
        for ( uint i = 0; i < _dim; i++ )
            val[i] = rand( max );
    }

    // return next element in sequence (special version for dim = 3)
    void next ( TVector & v, double max = 1.0 )
    {
        for ( uint i = 0; i < 3; i++ )
            v[i] = rand( max );
    }
    
    // special version for dim = 1 (and it's abbr.)
    double rand ( double max = 1.0 );
    double operator () ( double max = 1.0 ) { return rand( max ); }

    // initialise rng by single seed (seed must not equal 0)
    void init ( uint dim, ulong seed );

    // initialise rng by array
    void init ( uint dim, ulong init_key[], ulong length );

protected:
    // set next state in RNG
    void next_state ();
};

}// namespace

#endif  // __TRANDOM_HH
