#ifndef __TSPHERE_HH
#define __TSPHERE_HH
//
// Project : lib3D
// File    : TSphere.hh
// Purpose : class representing a sphere
// Author  : Ronald Kriemann
//
// arch-tag: bc8828ed-6653-4182-b332-6b9b02e0475b
//

#include "TGraphObject.hh"

namespace RGL
{

//
//
//
class TSphere : public TGraphObject
{
protected:
    // data specifying a sphere
    TVector   _center;
    real      _radius;

public:
    //////////////////////////////////////////////
    //
    // constuctor and destructor
    //

    TSphere ();
    TSphere ( const TVector & c, real r );
    TSphere ( const TVector & c, const TVector & r );

    virtual ~TSphere ();

    //////////////////////////////////////////////////
    //
    // access internal data
    //

    const TVector & center () const { return _center; }
    real            radius () const { return _radius; }
    
    //////////////////////////////////////////////////
    //
    // raytracing methods
    //

    // compute intersection with ray
    // - return true if intersected, else false
    virtual bool intersect ( const TRay & ray, real & dist ) const;

    // compute normal at point
    virtual bool calc_normal ( const TVector & point,
                               TVector       & normal ) const;

    //////////////////////////////////////////////////
    //
    // misc. methods
    //

    // store bounding box defined by xy, xz and yz planes
    // in given vectors
    virtual bool bounding_box ( TVector & min, TVector & max ) const;

    // print object
    virtual void print () const;
    
    //////////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TSphere * clone () const { return new TSphere; }
    virtual TSphere * copy  () const;
};

} // namespace

#endif  // __TSPHERE_HH
