#ifndef __TSTRING_HH
#define __TSTRING_HH
//
// Project : 
// File    : TString.hh
// Purpose : class for a string
// Author  : Ronald Kriemann
//
// arch-tag: 8ce99262-b6cf-4537-afec-469d01c9ad23
//

#include <limits.h>
#include <sys/types.h>

#include <iostream>

#include "TArray.hh"
#include "defines.hh"

namespace RGL
{

//
// class for a dynamic string (char-array)
//
class TString : public TArray< char >
{
public:
    //
    // constructor and destructor
    //

    TString ( uint size = 0 );
    TString ( const TString & string );
    TString ( const char * string, int size = -1 );

    virtual ~TString () { set_size( 0 ); }

    //
    // access string status and data
    //
    
    virtual uint length () const;

    virtual char       * c_str ()       { return c_array(); }
    virtual const char * c_str () const { return c_array(); }
    
    //
    // string handling
    //

    // copy content of string to buffer
    virtual void copy ( char * buffer, uint size ) const;

    // set string by buffer content
    virtual void set ( const char * buffer, int size = -1 );

    // access substrings [pos,pos+len) and [pos,length)
    virtual TString substr ( uint pos, uint len ) const;
    virtual TString index  ( uint pos ) const;

    // locate substrings or char.
    virtual TString strchr ( char c ) const;
    virtual TString strstr ( const TString & needle ) const;

    // split string into substrings seperated by delim
    virtual int split ( const TString & delim, TArray< TString > & substr );
    
    //
    // various operators for conversion and manipulation
    //

    virtual TString & operator = ( const TString & string );
    virtual TString & operator = ( const char    * buf ) { set( buf ); return *this; }
    virtual TString & operator = ( char            c   );

    virtual TString & operator += ( const TString & string );
    virtual TString & operator += ( char c );

    virtual bool operator == ( const TString & string ) const;
    virtual bool operator != ( const TString & string ) const { return ! (*this == string); }

    friend TString operator + ( const TString & str1, const TString & str2 )
    { TString  tmp( str1 ); return tmp += str2;
    }

    friend TString operator + ( const TString & str1, char c )
    { TString  tmp( str1 ); return tmp += c;
    }

    friend TString operator + ( char c, const TString & str )
    { TString  tmp; tmp = c; return tmp += str; }

    // convert characters in string to lower/upper
    TString & to_lower ();
    TString & to_upper ();
    
    //
    // misc. methods
    //

    // remove 0's at end
    virtual void shorten ();

    //
    // stream output
    //

    friend std::ostream & operator << ( std::ostream & os , const TString & string )
    { os << string.c_str(); return os; }
};

}// namespace

#endif  // __TSTRING_HH
