#ifndef __TTHINLENSCAMERA_HH
#define __TTHINLENSCAMERA_HH
//
// Project : lib3D
// File    : TThinLensCamera.hh
// Purpose : class representing a camera (viewing plane etc)
// Author  : Ronald Kriemann
//
// arch-tag: c3e7086d-aada-4073-8c4c-697f5ac4d134
//

#include "TCamera.hh"

namespace RGL
{

//
// put a thin lens in front of the image plane and
// model object focus 
//
class TThinLensCamera : public TCamera
{
protected:
    // distance of point of focus from origin
    // - this is also equal to the distance of the
    //   view plane from the origin
    // - if focus_dist = -1 => no focus
    real             _focus_dist;
    
    // radius of thin-lens around origin
    real             _tl_radius;

    // samples per point for DoF
    uint             _tl_samples;
    TArray< real >   _tl_samples_x, _tl_samples_y;
    
public:
    //////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TThinLensCamera ();

    virtual ~TThinLensCamera ();

    //////////////////////////////////////////////
    //
    // define camera
    //

    // define thin-lens of camera
    void set_thin_lens ( real aperture, real focus_len, uint dof_samples );
    
    //////////////////////////////////////////////
    //
    // setup ray corresponding to pixel
    // coordinates (i,j), i,j \in [0,1]
    //

    virtual void set_rays ( TArray< TRay > & rays, real i, real j ) const
    {
        rays.set_size( _tl_samples );
        
        for ( uint l = 0; l < _tl_samples; l++ )
        {
            rays[l].new_id();
            rays[l].set_initial();
            rays[l].origin().set( 1.0, _origin ).add( _tl_samples_x[l], _u,
                                                      _tl_samples_y[l], _v );
            rays[l].direction().set( 1.0, _ulc, -1.0, rays[l].origin() );
            rays[l].direction().add( i, _vp_right, j, _vp_down );
            rays[l].direction().normalise();
        }// for
    }
        
private:

    //
    // update internal data
    //

    virtual void update ();
};

} // namespace

#endif  // __TTHINLENSCAMERA_HH
