#ifndef __TTRIANGLE_HH
#define __TTRIANGLE_HH
//
// Project : lib3D
// File    : TTriangle.hh
// Purpose : class representing a triangle
// Author  : Ronald Kriemann
//
// arch-tag: 75eea06c-1ca1-4a8d-9b8f-409c544c3f31
//

#include "TGraphObject.hh"

namespace RGL
{

//
// class for a triangle, represented by 3 points
// - triangle is flat, e.g. normal is constant over area
// - default triangle is ( <0,0,0>, <1,0,0>, <0,1,0> )
// - orientation: v0, v1, v2
//
class TTriangle : public TGraphObject
{
protected:
    // corners of the triangle
    TVector   _v0, _v1, _v2;

    // plane equation of triangle
    TVector   _normal;
    real      _distance;

public:
    /////////////////////////////////////////////
    //
    // constructors
    //

    TTriangle ();

    TTriangle ( const TVector & v0,
                const TVector & v1,
                const TVector & v2 );

    virtual ~TTriangle () {}
    
    //////////////////////////////////////////////////
    //
    // misc. methods
    //

    // store bounding box defined by xy, xz and yz planes
    // in given vectors
    // - return false of bb is not defined (e.g. infinite object)
    virtual bool bounding_box ( TVector & min, TVector & max ) const;

    //////////////////////////////////////////////////
    //
    // raytracing methods
    //

    // compute intersection with ray
    // - return true if intersected, else false
    virtual bool intersect ( const TRay & ray, real & dist ) const;

    // compute normal at point
    virtual bool calc_normal ( const TVector & point,
                               TVector       & normal ) const
    {
        normal = _normal; return true;
    }

    //////////////////////////////////////////////////
    //
    // virtual constructors
    //

    virtual TTriangle * clone () const { return new TTriangle( _v0, _v1, _v2 ); }
    virtual TTriangle * copy  () const { return (TTriangle*) TGraphObject::copy(); }

protected:
    //////////////////////////////////////////////////
    //
    // misc.
    //

    // compute plane-equation of triangle
    void comp_plane_eq ();
};

}// namespace

#endif  // __TTRIANGLE_HH
