#ifndef __TWORLD_HH
#define __TWORLD_HH
//
// Project : lib3D
// File    : TWorld.hh
// Purpose : Declaration of a class for managing all graphical objects
// Author  : Ronald Kriemann
//
// arch-tag: 813d4f1f-d4ac-4ec4-b2c0-ec74bad62f2c
//

#include "TSLL.hh"

#include "TGraphObject.hh"

namespace RGL
{

// forward decl.
class TLight;

//
// objects are stored in list and in octree
//
class TWorld
{
public:
    //////////////////////////////////////////////////
    //
    // local types
    //

    typedef  TSLL< TGraphObject * >  TObjList;
    typedef  TObjList::TIterator     TObjIterator;

    typedef  TSLL< TLight * >        TLightList;
    typedef  TLightList::TIterator   TLightIterator;
    
protected:
    // list of all objects in this world
    TObjList      _obj_list;

    // all object, which have no bounding box
    TObjList      _inf_obj;

    // list of lights in our world
    TLightList    _lights;
    
public:
    //////////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TWorld ();

    ~TWorld ();

    //////////////////////////////////////////////////
    //
    // manage objects
    //

    // access to object-list
    const TObjList & objects () const { return _obj_list; }
    TObjList &       objects ()       { return _obj_list; }
    
    // add a new object to the world
    void add ( TGraphObject * obj );

    // access lights
    const TLightList & lights () const { return _lights; }
    TLightList &       lights ()       { return _lights; }

    // add a light to the world
    void add_light ( TLight * light );
    
    // transform all objects (only for test purpose)
    void transform ();
};

} // namespace

#endif  // __TWORLD_HH
