#ifndef __TX11PRINTER_HH
#define __TXIMAGE_HH
//
// Project : PHI
// File    : TXImage.hh
// Purpose : shows an image in an X11 window
// Author  : Ronald Kriemann
//
// arch-tag: d4d4d655-bb69-4112-997d-772ba497620f
//

#include "TArray.hh"
#include "TString.hh"
#include "TImage.hh"

namespace RGL
{

// internal X11 related data (forward decl.)
class TX11Data;
    
//
// write internal image to X11 window
//
class TXImage : public TImage
{
protected:
    // true if printer is active, e.g. ready to print
    bool                        _active;

    // position for moveto/lineto
    int                         _old_pos[2];
    
    // X11 related data
    TX11Data                  * _x11data;

    // for direct colour access in truecolor mode
    bool                        _truecolour;
    ulong                       _col_mask[3];
    int                         _col_shift[3];
    
    // true if window has to be finished
    bool                        _finish;

    // true if it should be drawn to window
    bool                        _draw_win;

    // available fonts
    TArray< TArray< TString > > _fonts;
    uint                        _old_type;
    int                         _old_size;

    // gamma correction factor
    real                        _gamma;
    
public:
    //////////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TXImage ( int w, int h, real gamma = 1.0 );

    virtual ~TXImage ();

    //////////////////////////////////////////////////
    //
    // access local data
    //

    void set_gamma ( real gamma ) { _gamma = gamma; }
    
    //////////////////////////////////////////////////
    //
    // methods for image drawing
    //

    // draw a point
    virtual void draw_point ( uint x, uint y, const TColour & c );
    
    // fill image with colour
    virtual void fill ( const TColour & c );

    // copy given image at given position
    virtual void bitblt ( uint x, uint y, const TImage & img );
    
    //////////////////////////////////////////////////
    //
    // methods for X11 drawing
    //

    // begin and end drawing
    virtual void begin ();
    virtual void end   ();

    //
    // drawing primitives
    //

    virtual void draw_point ( int x, int y );
    virtual void draw_line  ( int x1, int y1, int x2, int y2 );

    virtual void draw_rect ( int x1, int y1, int x2, int y2 );
    
    virtual void fill_rect ( int x1, int y1, int x2, int y2 );
    
    virtual void draw_circle ( int x, int y, int radius );
    virtual void fill_circle ( int x, int y, int radius );
    
    virtual void move_to ( int x, int y );
    virtual void line_to ( int x, int y );

    virtual void draw_text( int x, int y, const char * text );

    //
    // set styles
    //

    virtual void set_rgb        ( int r, int g, int b );
    virtual void set_pen        ( uint type );
    virtual void set_line_width ( int width );
    virtual void set_font       ( const char * font, int size );

protected:
    //////////////////////////////////////////////////
    //
    // misc.
    //

    // setup X11 window
    void init ();

    // determine available fonts
    void setup_fonts ();
    
    // handle pending events
    void handle_events ();

    // internal drawing method
    void draw ();
};

}// namespace

#endif  // __TXIMAGE_HH
