#ifndef __DEFINES_HH
#define __DEFINES_HH
//
// Project : 
// File    : defines.hh
// Purpose : common definitions
// Author  : Ronald Kriemann
//
// arch-tag: 8828f5e6-478d-4468-915d-b8425d1103f7
//

//
// type declarations
//
typedef  unsigned char   uchar;
typedef  unsigned int    uint;
typedef  unsigned long   ulong;
typedef  double          real;

//
// tolerance for floating-point-arithmetic
//
#define EPSILON  real(1e-8)

//
// functions
//

// minimum/maximum
template< class T > T min  ( const T & a, const T & b ) { return (a < b) ? a : b; }
template< class T > T max  ( const T & a, const T & b ) { return (a > b) ? a : b; }

// restrict value to interval [a,b]
template <class T > T min_max ( const T & val, const T & a, const T & b )
{ return min( max( val, a ), b ); }
      
// swap values
template< class T > void swap ( T & a, T & b ) { T tmp = a; a = b; b = tmp; }

//
// verbosity handling
//

extern unsigned int  verbosity;

inline bool verbose ( unsigned int v ) { return (v <= verbosity); }
void set_verbosity ( unsigned int v );

#endif			// __DEFINES_HH
