#ifndef __UFO_HH
#define __UFO_HH
//
// Project : 
// File    : ufo.hh
// Purpose : some functions for _u_ser_f_riendly_o_utput
// Author  : Ronald Kriemann
//
// arch-tag: d4e7f57e-524b-4c5e-95c9-ebb704cb64f8
//

#include <stdio.h>

#include "TString.hh"

namespace RGL
{
    
/////////////////////////////////////////////////////////////////
//
// take given number of bytes and return readable string 
//
/////////////////////////////////////////////////////////////////

class TByteOutput
{
protected:
    // short or lonf output
    static int _verbosity;
    
public:
    // set output styles
    void set_verbosity ( int v ) { _verbosity = v; }
    void inc_verbosity ()        { _verbosity++;   }
    void dec_verbosity ()        { _verbosity--;   }
    
    // byte -> string
    TString operator () ( unsigned long count ) const;
};

// object for easier usage
extern TByteOutput byte_out;

/////////////////////////////////////////////////////////////////
//
// take argument(s) and convert to string in printf-like style
//
/////////////////////////////////////////////////////////////////

template <class T>
TString
to_str ( const char * style, T n )
{                                                  
    TString  tmp;
    
    if ( style == NULL )
        return tmp;
    
    char buffer[256];
    
    snprintf( buffer, sizeof(buffer), style, n );
    tmp = buffer;
    
    return tmp;
}

template <class T1, class T2>
TString
to_str ( const char * style, T1 n1, T2 n2 )
{                                                  
    TString  tmp;
    
    if ( style == NULL )
        return tmp;
    
    char buffer[256];
    
    snprintf( buffer, sizeof(buffer), style, n1, n2 );
    tmp = buffer;
    
    return tmp;
}

template <class T1, class T2, class T3>
TString
to_str ( const char * style, T1 n1, T2 n2, T3 n3 )
{                                                  
    TString  tmp;
    
    if ( style == NULL )
        return tmp;
    
    char buffer[256];
    
    snprintf( buffer, sizeof(buffer), style, n1, n2, n3 );
    tmp = buffer;
    
    return tmp;
}

/////////////////////////////////////////////////////////////////
//
// colourful output
//
/////////////////////////////////////////////////////////////////

class TColourOut
{
protected:
    // colour string
    TString  _col;

public:
    // access colour string
    const TString & colour_str     () const                { return _col; }
    void            set_colour_str ( const TString & col ) { _col = col; }
    
    // colorise argument
    TString operator() ( const TString & str ) const;

    // stream operator
    friend std::ostream & operator << ( std::ostream & os, const TColourOut & c );
};

// output colours
extern TColourOut NORMAL;
extern TColourOut BOLD;
extern TColourOut RED;
extern TColourOut GREEN;
extern TColourOut BLUE;

} // namespace

#endif  // __UFO_HH
