//
// Project : lib3D
// File    : TLight.cc
// Purpose : baseclass for all lights
// Author  : Ronald Kriemann
//
// arch-tag: bd4cbc37-d726-4c27-8be3-98eb83ca6799
//

#include "TLight.hh"

namespace RGL
{

//////////////////////////////////////////////
//
// compute coefficient for different kind
// of object-light interaction
//

//
// compute diffuse reflection coefficient
// f_diff = c (L . N) ^ b
//   c : objects diffuse coeffcient
//   L : unit vector form surface point to light
//   N : normal vector at surface point
//   b : objects brilliance
//
real
TLight::comp_diffuse ( const TGraphObject * obj,
                       const TVector      & normal,
                       const TVector      & l ) const
{
    real angle  = dot( l, normal );
    real f_diff = 0.0;
    
    if (angle > 0.0)
    {
/*        if ( obj->texture().brilliance() != 1.0 )
            f_diff = std::pow( angle, obj->texture().brilliance() );
            else */
            f_diff = angle;
    }// if

    f_diff *= obj->texture()->diffuse();

    return f_diff;
}

//
// compute specular reflection coefficient
// f_spec = c ( R . V ) ^ d
//   c : objects specular coefficient
//   R : reflected unit vector of L (see above)
//   V : unit vector form surface point to camera (viewer)
//   d : specular degree
//   
real
TLight::comp_specular ( const TGraphObject * obj,
                        const TVector      & normal,
                        const TVector      & l,
                        const TVector      & v ) const
{
    TVector  r;
    real     angle;
    real     f_spec = 0.0;

    r.set( 2.0 * dot( normal, l ), normal, -1.0, l ).normalise();
    angle = dot( r, v );

    if ( angle > 0.0 )
    {
        if ( obj->texture()->specular_degree() != 1.0 )
            f_spec = std::pow( angle, obj->texture()->specular_degree() );
        else
            f_spec = angle;
    }// if

    f_spec *= obj->texture()->specular();

    return f_spec;
}

} // namespace
