//
// Project : lib3D
// File    : TMatrix.cc
// Purpose : Declaration of a class for a 4x4 matrix (used in 3d-transformation)
// Author  : Ronald Kriemann
//
// arch-tag: 4ceb26b6-75d4-4970-ba26-d04de5a6b7dd
//

#include <stdio.h>

#include "ufo.hh"

#include "TMatrix.hh"

namespace RGL
{

// define for usual matrix-access
#define M(i,j)  _data[ (i << 2) + j ]

//
// matrix multiplication from right
//
TMatrix &
TMatrix::mul_right ( const TMatrix & m )
{
    real      s;
    TMatrix   tmp;

    for ( uint i = 0; i < 3; i++ )
    {
        for ( uint j = 0; j < 4; j++ )
        {
            s = 0;
            
            for ( uint k = 0; k < 3; k++ )
                s += M(i,k) * m.M(k,j);

            tmp.M(i,j) = s;
        }// for
    }// for
    
    // append last column
    tmp.M(0,3) += M(0,3);
    tmp.M(1,3) += M(1,3);
    tmp.M(2,3) += M(2,3);

    return *this = tmp;
}

//
// matrix multiplication from left
//
TMatrix &
TMatrix::mul_left ( const TMatrix & m )
{
    real      s;
    TMatrix   tmp;

    for ( uint i = 0; i < 3; i++ )
    {
        for ( uint j = 0; j < 4; j++ )
        {
            s = 0;
            
            for ( uint k = 0; k < 3; k++ )
                s += m.M(i,k) * M(k,j);

            tmp.M(i,j) = s;
        }// for
    }// for
    
    // append last column
    tmp.M(0,3) += m.M(0,3);
    tmp.M(1,3) += m.M(1,3);
    tmp.M(2,3) += m.M(2,3);

    return *this = tmp;
}

//
// invert matrix: C = this^-1
//
void
TMatrix::invert ( TMatrix & C ) const
{
    //
    // unrolled Gaussian elimination
    //

    C = *this;
    
    C(0,0) = 1.0 / C(0,0);
    C(0,1) = C(0,0) * C(0,1);
    C(0,2) = C(0,0) * C(0,2);
    C(0,3) = C(0,0) * C(0,3);
    C(1,1) = C(1,1) - C(1,0) * C(0,1);
    C(1,2) = C(1,2) - C(1,0) * C(0,2);
    C(1,3) = C(1,3) - C(1,0) * C(0,3);
    C(2,1) = C(2,1) - C(2,0) * C(0,1);
    C(2,2) = C(2,2) - C(2,0) * C(0,2);
    C(2,3) = C(2,3) - C(2,0) * C(0,3);
    C(1,0) = - C(1,0) * C(0,0);
    C(2,0) = - C(2,0) * C(0,0);
    C(1,1) = 1.0 / C(1,1);
    C(1,2) = C(1,1) * C(1,2);
    C(1,3) = C(1,1) * C(1,3);
    C(1,0) = C(1,1) * C(1,0);
    C(2,2) = C(2,2) - C(2,1) * C(1,2);
    C(2,3) = C(2,3) - C(2,1) * C(1,3);
    C(2,0) = C(2,0) - C(2,1) * C(1,0);
    C(2,1) = - C(2,1) * C(1,1);
    C(2,2) = 1.0 / C(2,2);
    C(2,3) = C(2,2) * C(2,3);
    C(2,0) = C(2,2) * C(2,0);
    C(2,1) = C(2,2) * C(2,1);
    C(2,3) = - C(2,3);
    C(1,2) = 0.0;
    C(1,3) = - C(1,3);
    C(0,1) = 0.0;
    C(0,2) = 0.0;
    C(0,3) = - C(0,3);
    C(1,0) = C(1,0) - C(1,2) * C(2,0);
    C(1,1) = C(1,1) - C(1,2) * C(2,1);
    C(1,2) = C(1,2) - C(1,2) * C(2,2);
    C(1,3) = C(1,3) - C(1,2) * C(2,3);
    C(0,0) = C(0,0) - C(0,2) * C(2,0);
    C(0,1) = C(0,1) - C(0,2) * C(2,1);
    C(0,2) = C(0,2) - C(0,2) * C(2,2);
    C(0,3) = C(0,3) - C(0,2) * C(2,3);
    C(0,0) = C(0,0) - C(0,1) * C(1,0);
    C(0,1) = C(0,1) - C(0,1) * C(1,1);
    C(0,2) = C(0,2) - C(0,1) * C(1,2);
    C(0,3) = C(0,3) - C(0,1) * C(1,3);
}

//
// this = this^-1
//
TMatrix &
TMatrix::invert ()
{
    //
    // unrolled Gaussian elimination
    //
    
    M(0,0) = 1.0 / M(0,0);
    M(0,1) = M(0,0) * M(0,1);
    M(0,2) = M(0,0) * M(0,2);
    M(0,3) = M(0,0) * M(0,3);
    M(1,1) = M(1,1) - M(1,0) * M(0,1);
    M(1,2) = M(1,2) - M(1,0) * M(0,2);
    M(1,3) = M(1,3) - M(1,0) * M(0,3);
    M(2,1) = M(2,1) - M(2,0) * M(0,1);
    M(2,2) = M(2,2) - M(2,0) * M(0,2);
    M(2,3) = M(2,3) - M(2,0) * M(0,3);
    M(1,0) = - M(1,0) * M(0,0);
    M(2,0) = - M(2,0) * M(0,0);
    M(1,1) = 1.0 / M(1,1);
    M(1,2) = M(1,1) * M(1,2);
    M(1,3) = M(1,1) * M(1,3);
    M(1,0) = M(1,1) * M(1,0);
    M(2,2) = M(2,2) - M(2,1) * M(1,2);
    M(2,3) = M(2,3) - M(2,1) * M(1,3);
    M(2,0) = M(2,0) - M(2,1) * M(1,0);
    M(2,1) = - M(2,1) * M(1,1);
    M(2,2) = 1.0 / M(2,2);
    M(2,3) = M(2,2) * M(2,3);
    M(2,0) = M(2,2) * M(2,0);
    M(2,1) = M(2,2) * M(2,1);
    M(2,3) = - M(2,3);
    M(1,2) = 0.0;
    M(1,3) = - M(1,3);
    M(0,1) = 0.0;
    M(0,2) = 0.0;
    M(0,3) = - M(0,3);
    M(1,0) = M(1,0) - M(1,2) * M(2,0);
    M(1,1) = M(1,1) - M(1,2) * M(2,1);
    M(1,2) = M(1,2) - M(1,2) * M(2,2);
    M(1,3) = M(1,3) - M(1,2) * M(2,3);
    M(0,0) = M(0,0) - M(0,2) * M(2,0);
    M(0,1) = M(0,1) - M(0,2) * M(2,1);
    M(0,2) = M(0,2) - M(0,2) * M(2,2);
    M(0,3) = M(0,3) - M(0,2) * M(2,3);
    M(0,0) = M(0,0) - M(0,1) * M(1,0);
    M(0,1) = M(0,1) - M(0,1) * M(1,1);
    M(0,2) = M(0,2) - M(0,1) * M(1,2);
    M(0,3) = M(0,3) - M(0,1) * M(1,3);

    return *this;
}

void
TMatrix::invert3 ( TMatrix & C ) const
{
    C = *this;
    
    C(0,0) = 1.0 / C(0,0);
    C(0,1) = C(0,0) * C(0,1);
    C(0,2) = C(0,0) * C(0,2);
    C(1,1) = C(1,1) - C(1,0) * C(0,1);
    C(1,2) = C(1,2) - C(1,0) * C(0,2);
    C(2,1) = C(2,1) - C(2,0) * C(0,1);
    C(2,2) = C(2,2) - C(2,0) * C(0,2);
    C(1,0) = - C(1,0) * C(0,0);
    C(2,0) = - C(2,0) * C(0,0);
    C(1,1) = 1.0 / C(1,1);
    C(1,2) = C(1,1) * C(1,2);
    C(1,0) = C(1,1) * C(1,0);
    C(2,2) = C(2,2) - C(2,1) * C(1,2);
    C(2,0) = C(2,0) - C(2,1) * C(1,0);
    C(2,1) = - C(2,1) * C(1,1);
    C(2,2) = 1.0 / C(2,2);
    C(2,0) = C(2,2) * C(2,0);
    C(2,1) = C(2,2) * C(2,1);
    C(1,0) = C(1,0) - C(1,2) * C(2,0);
    C(1,1) = C(1,1) - C(1,2) * C(2,1);
    C(1,2) = - C(1,2) * C(2,2);
    C(0,0) = C(0,0) - C(0,2) * C(2,0);
    C(0,1) = - C(0,2) * C(2,1);
    C(0,2) = - C(0,2) * C(2,2);
    C(0,0) = C(0,0) - C(0,1) * C(1,0);
    C(0,1) = C(0,1) - C(0,1) * C(1,1);
    C(0,2) = C(0,2) - C(0,1) * C(1,2);
}

///////////////////////////////////////////////
//
// set matrix to various transformations
//

//
// build scaling by factor <sx, sy, sz>
//
TMatrix &
TMatrix::set_scale ( const TVector & s )
{
    _data[0] = s.x(); _data[1] = 0.0;   _data[2]  = 0.0;   _data[3]  = 0.0; 
    _data[4] = 0.0;   _data[5] = s.y(); _data[6]  = 0.0;   _data[7]  = 0.0; 
    _data[8] = 0.0;   _data[9] = 0.0;   _data[10] = s.z(); _data[11] = 0.0;
    
    return *this;
}

//
// build translation by vector <tx, ty, tz>
//
TMatrix &
TMatrix::set_translate ( const TVector & t )
{
    _data[0] = 1.0; _data[1] = 0.0; _data[2]  = 0.0; _data[3]  = t.x(); 
    _data[4] = 0.0; _data[5] = 1.0; _data[6]  = 0.0; _data[7]  = t.y(); 
    _data[8] = 0.0; _data[9] = 0.0; _data[10] = 1.0; _data[11] = t.z();

    return *this;
}

//
// build rotation around v and angle alpha
//
TMatrix &
TMatrix::set_rotate ( const TVector & v, real alpha )
{
    //
    // rotation by alpha around v is given by
    //
    //   | x^2+(1-x^2) cos a        xy(1-cos a) + z sin a     xz(1-cos a) - y sin a |
    //   | xy(1-cos a) - z sin a    y^2 + (1-y^2) cos a       yz(1-cos a) + x sin a |
    //   | xz(1-cos a) + y sin a    yz(1-cos a) - x sin a     z^2 + (1-z^2) cos a   |
    //
    
    const real  c   = std::cos( alpha );
    const real  s   = std::sin( alpha );
    const real  xx  = v.x() * v.x();
    const real  yy  = v.y() * v.y();
    const real  zz  = v.z() * v.z();
    const real  omc = 1.0 - c;

    _data[0]  = xx + c * ( 1.0 - xx );
    _data[1]  = v.x() * v.y() * omc - v.z() * s;
    _data[2]  = v.z() * v.x() * omc + v.y() * s;
    _data[3]  = 0.0;

    _data[4]  = v.x() * v.y() * omc + v.z() * s;
    _data[5]  = yy + c * ( 1.0 - yy );
    _data[6]  = v.y() * v.z() * omc - v.x() * s;
    _data[7]  = 0.0;

    _data[8]  = v.z() * v.x() * omc - v.y() * s;
    _data[9]  = v.y() * v.z() * omc + v.x() * s;
    _data[10] = zz + c * ( 1.0 - zz );
    _data[11] = 0.0;

    return *this;
}
    
//
// stream output
//
std::ostream &
operator << ( std::ostream & os, const TMatrix & m )
{
    for ( uint i = 0; i < 3; i++ )
    {
        for ( uint j = 0; j < 3; j++ )
            os << to_str( "%5.3f    ", (float) m(i,j) );

        if (i < 3)
            os << to_str( "%5.3f", (float) m(i,3) ) << std::endl;
        else
            os << to_str( "%5.3f", (float) m(i,3) );
    }// for

    return os;
}

}// namespace
