%{
    //
    // C-declarations
    //
    // arch-tag: 6ade5ea9-c6df-47c6-adbb-a78cbd1f9704
    //

    #include <iostream>
    
    #include "TParser.hh"

    extern RGL::TParser * parser;

    extern FILE* yyin;
    
    extern int yylex();
    
    void yyerror ( const char * s );

    // for debugging
    int text_row;
    int text_col;

    // output
    //#define PRINT( str )  { printf( str ); printf( "\n" ); }
    #define PRINT( str )  
 %}

%union {
    double                  num_val;
    bool                    bool_val;
    char                    str_val[256];
    RGL::TParser::TNode   * node;
}

%token<num_val>  NUM
%token<bool_val> BOOLEAN
%token<str_val>  NAME STRING
%token WHILE IF ELSE
%token CONST

%right '=' ADDASSIGN SUBASSIGN
%left AND OR
%right NOT
%nonassoc  '<' '>' LE GE EQ NE
%left '|'
%left  '+' '-' INCREMENT DECREMENT
%left  '*' '/'
%left  '('
%right NEG

%type<node>  node block stmts stmt expr
%type<node>  name
%type<node>  data object funccall vector arglist carglist

%%

program : stmts { PRINT("p : ss"); parser->eval_tree( $1 ); }
        ;

stmts :            { PRINT("ss : NULL"); $$ = NULL; }
      | stmts stmt { PRINT("ss : ss s"); $$ = parser->append_stmt( $1, $2 ); }
      ;

stmt  : node ';'                         { $$ = parser->new_stmt( $1 ); }
      | WHILE '(' expr ')' block         { $$ = parser->new_while( $3, $5 ); }
      | IF '(' expr ')' block            { $$ = parser->new_if( $3, $5, NULL ); }
      | IF '(' expr ')' block ELSE block { $$ = parser->new_if( $3, $5, $7 ); }
      ;

block : stmt          { PRINT("b : s");      $$ = parser->new_block( $1 ); } 
      | '{' stmts '}' { PRINT("b : { ss }"); $$ = parser->new_block( $2 ); }
      ;

node :      { PRINT("n : NULL"); $$ = NULL; }
     | expr { PRINT( "n : e ;" ); $$ = $1; }
     ;

expr  : expr '+' expr { PRINT("e : e + e"); $$ = parser->new_node( RGL::OP_ADD, $1, $3 ); }
      | expr '-' expr { PRINT("e : e - e"); $$ = parser->new_node( RGL::OP_SUB, $1, $3 ); }
      | expr '*' expr { PRINT("e : e * e"); $$ = parser->new_node( RGL::OP_MUL, $1, $3 ); }
      | expr '/' expr { PRINT("e : e / e"); $$ = parser->new_node( RGL::OP_DIV, $1, $3 ); }
      | '-' expr %prec NEG { PRINT("e : - e"); $$ = parser->new_node( RGL::OP_NEG, $2 ); }
      | '(' expr ')'  { PRINT("e : (e)"); $$ = $2; }
      | name '=' expr { PRINT("e : name = e"); $$ = parser->new_node( RGL::OP_ASSIGN, $1, $3 ); }
      | CONST name '=' expr { PRINT("e : const name = e"); parser->set_constant( $2 );
                              $$ = parser->new_node( RGL::OP_ASSIGN, $2, $4 ); }
      | name ADDASSIGN expr { PRINT("e : name += e"); $$ = parser->new_node( RGL::OP_ADDASSIGN, $1, $3 ); } 
      | name SUBASSIGN expr { PRINT("e : name -= e"); $$ = parser->new_node( RGL::OP_SUBASSIGN, $1, $3 ); }
      | INCREMENT expr { $$ = parser->new_node( RGL::OP_PREINC, $2 ); }
      | expr INCREMENT { $$ = parser->new_node( RGL::OP_POSTINC, $1 ); }
      | DECREMENT expr { $$ = parser->new_node( RGL::OP_PREDEC, $2 ); }
      | expr DECREMENT { $$ = parser->new_node( RGL::OP_POSTDEC, $1 ); }
      | expr EQ expr  { PRINT("e : e < e"); $$ = parser->new_node( RGL::OP_EQ, $1, $3 ); }
      | expr NE expr  { PRINT("e : e < e"); $$ = parser->new_node( RGL::OP_NE, $1, $3 ); }
      | expr '<' expr { PRINT("e : e < e"); $$ = parser->new_node( RGL::OP_LT, $1, $3 ); }
      | expr '>' expr { PRINT("e : e < e"); $$ = parser->new_node( RGL::OP_GT, $1, $3 ); }
      | expr LE expr  { PRINT("e : e < e"); $$ = parser->new_node( RGL::OP_LE, $1, $3 ); }
      | expr GE expr  { PRINT("e : e < e"); $$ = parser->new_node( RGL::OP_GE, $1, $3 ); }
      | expr AND expr { PRINT("e : e and e"); $$ = parser->new_node( RGL::OP_AND, $1, $3 ); }
      | expr OR expr  { PRINT("e : e or e");  $$ = parser->new_node( RGL::OP_OR, $1, $3 ); }
      | NOT expr      { PRINT("e : not e");   $$ = parser->new_node( RGL::OP_NOT, $2 ); }
      | data          { PRINT("e : data"); $$ = $1; }
      | name          { PRINT("e : name"); $$ = $1; }
      | object        { PRINT("e : obj"); $$ = $1; }
      | funccall      { PRINT("e : obj"); $$ = $1; }
      ;

name : NAME { PRINT( "name : var" ); $$ = parser->name_to_var( $1 ); }
     ;

data : NUM     { PRINT("data : number"); $$ = parser->new_datanode( RGL::TYPE_NUMBER, & $1 ); }
     | BOOLEAN { PRINT("data : bool");   $$ = parser->new_datanode( RGL::TYPE_BOOL, & $1 ); }
     | STRING  { PRINT("data : string"); $$ = parser->new_datanode( RGL::TYPE_STRING, $1 ); }
     ;

funccall : NAME '(' arglist ')' { $$ = parser->new_funccall( $1, $3 ); }
         | '|' expr '|'         { $$ = parser->new_node( RGL::OP_NORM2, $2 ); }
         ;

arglist :          { PRINT("al : ()");  $$ = parser->append_arg( NULL, NULL ); }
        | carglist { PRINT("al : cal"); $$ = $1; }
        ;

carglist : expr              { PRINT("cal : e");      $$ = parser->append_arg( NULL, $1 ); }
         | carglist ',' expr { PRINT("cal : al , e"); $$ = parser->append_arg( $1, $3 ); }
         ;

object : vector { PRINT( "object : vector" ); $$ = $1; }
       ;

vector : '<' expr ',' expr ',' expr '>' { $$ = parser->new_vector( $2, $4, $6 ); }
       ;

%%

int
parse_file ( RGL::TParser * p )
{
    parser = p;

    text_row = text_col = 1;
    
    return yyparse();
}

//
// Errorhandling
//
void 
yyerror ( const char * s )
{
    std::cout << "(TParser) ERROR : ";

    if ( s != NULL )
        std::cout << s;

    std::cout << " at (" << text_row << ", " << text_col << ")" << std::endl;
    
    exit( -1 );
}
