%{
    //
    // arch-tag: 4a9a7842-6a36-490f-bb65-418d2dd83fe0
    //

    #include <string.h>

    #include "TParser.hh"
    
    #include "parser/parser.y.hh"
    
    extern int text_row;
    extern int text_col;

    int new_tr = 1;
    int new_tc = 1;
    
    void update_pos();

    extern RGL::TParser * parser;
    
    // output
    //#define PRINT( str )  { printf( str ); printf( "\n" ); }
    #define PRINT( str )

    // change reading in of content to scan
    #define YY_INPUT(buf,result,max_size) { result = parser->readbuf( buf, max_size ); }
%}

LETTER   [a-zA-Z]
NUMBER   [0-9]
COMMENT  "#"
SPACE    [ \t]

%%

"while" { update_pos(); return WHILE; }
"if"    { update_pos(); return IF; }
"else"  { update_pos(); return ELSE; }
"const" { update_pos(); return CONST; }

"true"  { update_pos(); yylval.bool_val = true; return BOOLEAN; }
"false" { update_pos(); yylval.bool_val = false; return BOOLEAN; }

"and" { update_pos(); PRINT("lexer(and)"); return AND; }
"or"  { update_pos(); PRINT("lexer(or)");  return OR; }
"not" { update_pos(); PRINT("lexer(not)"); return NOT; }

("-")?{NUMBER}+("."{NUMBER}+)?(("e"|"E")("+"|"-")?{NUMBER}+)? { PRINT( "number found" ); yylval.num_val = atof( yytext ); update_pos(); return NUM; }

{LETTER}+({LETTER}|{NUMBER}|"_")* { PRINT( "name found" ); strncpy( yylval.str_val, yytext, 256 ); update_pos(); return NAME; }

\".*\" { PRINT( "string found" ); strncpy( yylval.str_val, yytext, 256 ); update_pos(); return STRING; }

"<=" { update_pos(); return LE; }
">=" { update_pos(); return GE; }
"!=" { update_pos(); return NE; }
"==" { update_pos(); return EQ; }

"+=" { update_pos(); return ADDASSIGN; }
"-=" { update_pos(); return SUBASSIGN; }
"++" { update_pos(); return INCREMENT; }
"--" { update_pos(); return DECREMENT; }

"="|"+"|"-"|"*"|"/"|";"|","|"|" { update_pos(); return yytext[0]; }

"("|")"|"{"|"}"|"<"|">" { update_pos(); return yytext[0]; }

\n { update_pos(); }

({COMMENT}.*|{SPACE}+) { update_pos(); }

%%

/*
 * update scanner-position for debugging
 */
void
update_pos ()
{
    text_row = new_tr;
    text_col = new_tc;
    
    if ( yytext[0] == '\n' )
    {
        new_tc = 1;
        new_tr++;
    }
    else
        new_tc += strlen( yytext );
}

/*
 * to avoid linking against libfl
 */
#ifndef yywrap
int
yywrap ( void )
{
    return 1;
}
#endif

